import { InjectionToken, EventEmitter, Injectable, Inject, Output, defineInjectable, inject, Component, Input, Directive, ViewContainerRef, ComponentFactoryResolver, ViewChild, NgModule } from '@angular/core';
import { __decorate, __metadata } from 'tslib';
import { LoadingEnum, HeaderEnum } from '@faab/http-lib';
import { Observable } from 'rxjs';
import { TranslateModule } from '@ngx-translate/core';
import { CommonModule } from '@angular/common';
import { Updatable } from '@faab/base-lib';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const PAGINATION_OPTIONS_TOKEN = new InjectionToken('PAGINATION_OPTIONS_TOKEN');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const PaginationSortingDirectionEnum = {
    DESC: 'desc',
    ASC: 'asc',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationService {
    /**
     * @param {?} token_config
     */
    constructor(token_config) {
        this.token_config = token_config;
        this.change = new EventEmitter();
        this.config = new EventEmitter();
        /** @type {?} */
        const config_default = {
            sort: 'whenCreated',
            direction: PaginationSortingDirectionEnum.DESC,
            max: 20,
            filter_components: []
        };
        if (token_config) {
            if (typeof token_config.sort === 'string') {
                config_default.sort = token_config.sort;
            }
            if (typeof token_config.direction === 'string' && PaginationSortingDirectionEnum[token_config.direction.toUpperCase()]) {
                config_default.direction = (/** @type {?} */ (token_config.direction));
            }
            if (typeof token_config.max === 'number') {
                config_default.max = token_config.max;
            }
            if (Array.isArray(token_config.filter_components)) {
                config_default.filter_components = token_config.filter_components;
            }
        }
        this.filter_components = config_default.filter_components;
        this.config.emit(config_default);
    }
    /**
     * @param {?} event_type
     * @param {?} default_component
     * @return {?}
     */
    getComponentByEventType(event_type, default_component) {
        if (typeof event_type !== 'string'
            || !Array.isArray(this.filter_components)) {
            return default_component;
        }
        /** @type {?} */
        const filter_components_length = this.filter_components.length;
        for (let i = 0; i < filter_components_length; i++) {
            /** @type {?} */
            const filter_component = this.filter_components[i];
            if (!filter_component
                || typeof filter_component.EVENT_TYPE !== 'string'
                || filter_component.EVENT_TYPE !== event_type) {
                continue;
            }
            return filter_component;
        }
        return default_component;
    }
    /**
     * @return {?}
     */
    getAllComponents() {
        return (Array.isArray(this.filter_components)) ? this.filter_components : [];
    }
}
PaginationService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
PaginationService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [PAGINATION_OPTIONS_TOKEN,] }] }
];
PaginationService.propDecorators = {
    change: [{ type: Output }]
};
/** @nocollapse */ PaginationService.ngInjectableDef = defineInjectable({ factory: function PaginationService_Factory() { return new PaginationService(inject(PAGINATION_OPTIONS_TOKEN)); }, token: PaginationService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AvailableService {
    constructor() {
        this._available = true;
        this.change = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get available() {
        return this._available;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set available(value) {
        if (this._available === value) {
            return;
        }
        this._available = value;
        this.change.emit(value);
    }
    /**
     * @return {?}
     */
    toggle() {
        this.available = !this.available;
        return this.available;
    }
}
AvailableService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AvailableService.ctorParameters = () => [];
AvailableService.propDecorators = {
    change: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationTheadCellSortingComponent {
    /**
     * @param {?} paginationService
     */
    constructor(paginationService) {
        this.paginationService = paginationService;
        this.clickChangeDirection = new EventEmitter();
        this._direction = null;
        this._subscription_direction_change = this.paginationService.change.subscribe((key) => {
            if (key !== this.key) {
                this._direction = null;
            }
        });
    }
    /**
     * @return {?}
     */
    get direction() {
        return this._direction;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set direction(value) {
        this._direction = value;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.onInit();
    }
    /**
     * @return {?}
     */
    onInit() { }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.onDestroy();
    }
    /**
     * @return {?}
     */
    onDestroy() {
        this.checkSubscription(this._subscription_direction_change);
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onClick(e) {
        if (!(e instanceof MouseEvent)) {
            return;
        }
        debugger;
        e.stopPropagation();
        e.preventDefault();
        this.paginationService.change.emit(this.key);
        if (this._direction === PaginationSortingDirectionEnum.ASC) {
            this._direction = PaginationSortingDirectionEnum.DESC;
        }
        else {
            this._direction = PaginationSortingDirectionEnum.ASC;
        }
        this.clickChangeDirection.emit(this._direction);
    }
    /**
     * @return {?}
     */
    getStyleClass() {
        return (!this._direction) ? '' : this._direction;
    }
    /**
     * @return {?}
     */
    getDirectionTitle() {
        if (this.direction === null) {
            return PaginationSortingDirectionEnum.ASC;
        }
        return this.direction === PaginationSortingDirectionEnum.DESC ?
            PaginationSortingDirectionEnum.ASC : PaginationSortingDirectionEnum.DESC;
    }
    /**
     * @private
     * @param {?} subscription
     * @return {?}
     */
    checkSubscription(subscription) {
        if (!subscription || typeof subscription.unsubscribe !== 'function') {
            return;
        }
        subscription.unsubscribe();
    }
}
PaginationTheadCellSortingComponent.EVENT_TYPE = 'SORTING';
PaginationTheadCellSortingComponent.PARAM_NAME_DIRECTION = 'direction';
PaginationTheadCellSortingComponent.PARAM_NAME_SORT = 'sort';
PaginationTheadCellSortingComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-pagination-base-thead-cell-sorting',
                template: "<!-- ('pagination-lib.title_direction' | translate) + 'getDirectionTitle() | translate'\n  + (title | translate | lowercase)\n  + ' '\n  + (getDirectionTitle() | translate | lowercase)\" -->\n\n<div\n  [title]=\"'pagination-lib.title' | translate:{\n    title: title | translate,\n    direction: 'pagination-lib.direction-' + getDirectionTitle() | translate\n  }\"\n  [class]=\"getStyleClass()\"\n  (click)=\"onClick($event)\"\n>\n  <svg version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" x=\"0px\" y=\"0px\"\n       width=\"100%\"\n\n       height=\"100%\" viewBox=\"0 0 13 12\" enable-background=\"new 0 0 13 12\" xml:space=\"preserve\">\n\n<g id=\"Ellipse_6_copy_6_1_\" enable-background=\"new    \">\n\t<g id=\"Ellipse_6_copy_6\">\n\t\t<g>\n\t\t\t<path class=\"arrow-up\" fill-rule=\"evenodd\" clip-rule=\"evenodd\" fill=\"#CCCCCC\" d=\"M6.008,0.011\n\t\t\t\tL1.002,5.004h10.012L6.008,0.011z\"/>\n\t\t</g>\n\t</g>\n</g>\n    <g id=\"Ellipse_6_copy_6_1_\" enable-background=\"new    \">\n\t<g id=\"Ellipse_6_copy_6\">\n\t\t<g>\n\t\t\t<path class=\"arrow-down\" fill-rule=\"evenodd\" clip-rule=\"evenodd\" fill=\"#CCCCCC\"\n            d=\"M6.008,11.988l5.005-4.979H1.002L6.008,11.988z \"/>\n\t\t</g>\n\t</g>\n</g>\n\n</svg>\n\n</div>\n",
                providers: [PaginationService],
                styles: ["div{width:12px;height:12px;margin:-8px 3px 2px 0;cursor:pointer}.arrow-down,.arrow-up{fill:var(--colorFontLight)}.asc .arrow-up{fill:var(--colorFontTheadAccent)}.asc .arrow-down,.desc .arrow-up{fill:var(--colorFontLight)}.desc .arrow-down{fill:var(--colorFontTheadAccent)}"]
            }] }
];
/** @nocollapse */
PaginationTheadCellSortingComponent.ctorParameters = () => [
    { type: PaginationService }
];
PaginationTheadCellSortingComponent.propDecorators = {
    direction: [{ type: Input }],
    title: [{ type: Input }],
    key: [{ type: Input }],
    clickChangeDirection: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationEvent {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* class decorator */
/**
 * @template T
 * @return {?}
 */
function StaticImplements() {
    return (constructor) => {
        return constructor;
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
let PaginationTheadCellBaseComponent = class PaginationTheadCellBaseComponent {
    constructor() {
        this.key = '';
        this.sortable = false;
        this.changePaginationEvent = new EventEmitter();
    }
    /**
     * @param {?} event_data
     * @param {?} process_data
     * @return {?}
     */
    static updateEventData(event_data, process_data) {
        return 1;
    }
    /**
     * @param {?} param_item
     * @return {?}
     */
    static updateEventDataByParamItem(param_item) { }
    /**
     * @return {?}
     */
    static getParamItem() {
        return '';
    }
    /**
     * @return {?}
     */
    static generateRequestUrl() {
        return '';
    }
    /**
     * @param {?} key
     * @return {?}
     */
    static getValueByKey(key) { }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.onInit();
    }
    /**
     * @return {?}
     */
    onInit() { }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.onDestroy();
    }
    /**
     * @return {?}
     */
    onDestroy() { }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.afterViewInit();
    }
    /**
     * @return {?}
     */
    afterViewInit() { }
    /**
     * @param {?} direction
     * @return {?}
     */
    onChangeDirection(direction) {
        debugger;
        /** @type {?} */
        const event = new PaginationEvent();
        event.type = PaginationTheadCellSortingComponent.EVENT_TYPE;
        event.data = (/** @type {?} */ ({
            key: this.key,
            value: direction
        }));
        this.changePaginationEvent.emit(event);
    }
    /**
     * @protected
     * @param {?} subscription
     * @return {?}
     */
    checkSubscription(subscription) {
        if (!subscription || typeof subscription.unsubscribe !== 'function') {
            return;
        }
        subscription.unsubscribe();
    }
};
PaginationTheadCellBaseComponent.EVENT_TYPE = '';
PaginationTheadCellBaseComponent.PARAM_NAME = '';
PaginationTheadCellBaseComponent.propDecorators = {
    key: [{ type: Input }],
    title: [{ type: Input }],
    direction: [{ type: Input }],
    sortable: [{ type: Input }],
    changePaginationEvent: [{ type: Output }]
};
PaginationTheadCellBaseComponent = __decorate([
    StaticImplements(),
    __metadata("design:paramtypes", [])
], PaginationTheadCellBaseComponent);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationTheadCellPlainComponent extends PaginationTheadCellBaseComponent {
}
PaginationTheadCellPlainComponent.EVENT_TYPE = 'PLAIN';
PaginationTheadCellPlainComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-pagination-base-thead-cell-plain',
                template: "<div class=\"d-flex align-items-center\">\n  <ng-container *ngIf=\"sortable\">\n    <lib-pagination-base-thead-cell-sorting *ngIf=\"sortable\"\n      [title]=\"title\"\n      [key]=\"key\"\n      [direction]=\"direction\"\n      (clickChangeDirection)=\"onChangeDirection($event)\"\n    ></lib-pagination-base-thead-cell-sorting>\n  </ng-container>\n  <span>{{title | translate}}</span>\n</div>\n"
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const AttributePaginationFormatEnum = {
    STATUS: 'status',
    VIEW: 'view',
    DATE: 'date',
    DATE_DD_MM_YYYY: 'date::dd-MM-yyyy',
    DATE_DD_MM_YYYY_H_MM_S: 'date::dd-MM-yyyy HH:mm:ss',
    DATE_D_EEEE_H_MM: 'date::d, EEEE H:mm',
    LOCALIZED: 'localized',
    LOCALIZED_TEXTAREA: 'localizedTextarea',
    TEXTAREA: 'textarea',
    STRING: 'string',
    NUMBER: 'number',
    BOOLEAN: 'boolean',
    SELECT: 'select',
    CURRENCY: 'currency',
    ACCORDION: 'accordion',
    TAGS: 'tags',
    PASSWORD: 'password',
    IMAGE: 'image',
    ADDRESS: 'address',
    COLOR: 'color',
    BARCODE_TYPE: 'barcodeType',
    BROADCAST_MESSAGE_STATUS: 'broadcast-message-status',
    CUSTOM: 'custom',
    STRING_DATE: 'stringDate',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationLibComponent {
    /**
     * @param {?} http
     * @param {?} storage
     * @param {?} location
     * @param {?} router
     * @param {?} availableService
     * @param {?} activatedRoute
     * @param {?} language
     * @param {?} datePipe
     * @param {?} paginationService
     */
    constructor(http, storage, location, router, availableService, activatedRoute, language, datePipe, paginationService) {
        this.http = http;
        this.storage = storage;
        this.location = location;
        this.router = router;
        this.availableService = availableService;
        this.activatedRoute = activatedRoute;
        this.language = language;
        this.datePipe = datePipe;
        this.paginationService = paginationService;
        this.items_length = 0;
        this.page = 1;
        this.direction = PaginationSortingDirectionEnum.DESC;
        this.direction_default = PaginationSortingDirectionEnum.DESC;
        this.urlParams = [];
        this.visiblePagination = false;
        this.dateFieldParam = {
            from: null,
            to: null
        };
        this._sort = 'whenCreated';
        this._sort_default = 'whenCreated';
        this.max = 20;
        this.prev = { server: '', url: '' };
        this.changeColumnsActual = new EventEmitter();
        this.component_default = PaginationTheadCellPlainComponent;
        // tslint:disable-next-line: no-unused-expression
        this.paginationService.config;
        this.subscription_config = this.paginationService.config.subscribe((config_new) => {
            if (!config_new) {
                return;
            }
            this.config = config_new;
            this.setConfig();
        });
        this.available = this.storage.getAvailable();
        this._items = [];
    }
    /**
     * @return {?}
     */
    get available() {
        return this._available;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set available(value) {
        this._available = value;
    }
    /**
     * @param {?} _sort
     * @return {?}
     */
    set sort(_sort) {
        if (typeof _sort !== 'string') {
            return;
        }
        this._sort = _sort;
    }
    /**
     * @param {?} controller
     * @return {?}
     */
    set initialization(controller) {
        this.setInitialization(controller);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.onInit();
    }
    /**
     * @return {?}
     */
    onInit() {
        this.items_length = 0;
        this.countPage = 0;
        this.countPageWithoutFilters = 0;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.onDestroy();
    }
    /**
     * @return {?}
     */
    onDestroy() {
        this.checkSubscription(this._subscription_available);
        this.checkSubscription(this._subscription_activatedRoute_queryParams);
        this.checkSubscription(this._subscription_excel_export_start);
        this.checkSubscription(this._subscription_excel_export_ready);
    }
    /* onClickStopExportXlsx(e: Event) {
        if(!(e instanceof Event)) {
          return;
        }
        e.preventDefault();
        e.stopPropagation();
        this.export_xlsx_exit = true;
      } */
    /**
     * @param {?} changed_columns
     * @return {?}
     */
    onChangedColumnsActual(changed_columns) {
        /* if (!changed_columns || !Array.isArray(changed_columns.actual) || !Array.isArray(changed_columns.deleted)) {
          return;
        }
        this.changeColumnsActual.emit(changed_columns);
    
        const length_deleted: number = changed_columns.deleted.length,
          param_indexes: number[] = [];
    
        let filter_str = (this.filter) ? this.filter : '';
    
        const filters: string[] = (filter_str) ? filter_str.split(PaginationLibComponent.SEPARATOR_MANY_VALUE) : [];
    
        let length_filters: number = filters.length;
        // direction: string = undefined;
    
        for (let i = 0; i < length_deleted; i++) {
          const column: TableColumn = changed_columns.deleted[i];
          if (!column || !column.key || typeof column.key !== 'string') {
            continue;
          }
          const filter_name: string = this.getFilterName(column.key);
          if (filter_str.includes(filter_name)) {
            for (let j = 0; j < length_filters; j++) {
              const filter: string = filters[j];
              if (!filter || typeof filter !== 'string' || filter.substring(0, filter_name.length) !== filter_name) {
                continue;
              }
              filters.splice(j, 1);
              length_filters = filters.length;
              filter_str = filters.join(PaginationLibComponent.SEPARATOR_MANY_VALUE);
              break;
            }
          }
    
        }
    
        const keys: string[] = [],
          length_actual: number = changed_columns.actual.length;
        for (let i = 0; i < length_actual; i++) {
          const column: TableColumn = changed_columns.actual[i];
          if (!column || !column.key || typeof column.key !== 'string') {
            continue;
          }
          keys.push(column.key);
        }
        this.keys = keys;
        this.filter = filter_str;
        const params_new: string = this.genParams();
    
        if (this.urlParams[PaginationLibComponent.PARAM_FIELD] === params_new) {
          this.onPagerInit();
          return;
        }
    
        this.urlParams[PaginationLibComponent.PARAM_FIELD] = params_new;
        this.write();
        this.attributesInit();
    
        const params_object: Params = {};
        params_object[PaginationLibComponent.PARAM_FIELD] = params_new;
    
        this.router.navigate(['.'], {
          queryParams: params_object,
          relativeTo: this.activatedRoute
        }); */
    }
    /**
     * @param {?} key
     * @return {?}
     */
    isSortableField(key) {
        return (this.attributes && this.attributes[key] && typeof this.attributes[key].sortable === 'boolean')
            ? this.attributes[key].sortable : false;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getAttributeTitleByKey(key) {
        return (typeof key !== 'string'
            ||
                !this.attributes
            || !this.attributes[key]
            || typeof this.attributes[key].label !== 'string') ? '' : this.attributes[key].label;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getDirection(key) {
        return this._sort === key ? this.direction : null;
    }
    /* public getFilterName(key: string) {
        return (this.attributes[key].filter && this.attributes[key].filter['field'])
        ? this.attributes[key].filter['field'] : key;
      } */
    /**
     * @param {?} key
     * @return {?}
     */
    getPaginationTheadCellComponentData(key) {
        if (typeof key !== 'string') {
            return undefined;
        }
        /** @type {?} */
        const event_type = this.getColumnType(key);
        /** @type {?} */
        const component = this.paginationService.getComponentByEventType(event_type, this.component_default);
        if (!this.isValidComponent(component)) {
            return undefined;
        }
        /** @type {?} */
        const component_data = {
            key: key,
            title: this.getAttributeTitleByKey(key),
            direction: this.getDirection(key),
            sortable: this.isSortableField(key),
            event_type: this.getColumnType(key)
        };
        if (event_type !== this.component_default['EVENT_TYPE']) {
            component_data.value = component['getValueByKey'](key);
        }
        return component_data;
    }
    /* public getSelected(key: string): string {
        key = this.getFilterName(key);
        if (!this.filter) {
          return '';
        }
    
        let result = '';
        try {
          const filter = this.filter.split(PaginationLibComponent.SEPARATOR_MANY_VALUE);
          filter.forEach(val => {
            if (val.search(key) === 0) {
              result = val.split('=')[1];
              throw Error();
            }
          });
        } catch (e) {
          // skip error
        }
        return result;
      } */
    /* public getExportProgress(): number {
        const current_items: number = this._items_export.length,
          request_part: number = 98,
          after_request_part: number = 100 - request_part;
        if(!current_items || !this.count_items_export) {
          return 0;
        }
        if(this.count_items_export > current_items) {
          return Math.round((this._items_export.length / this.count_items_export) * request_part);
        }
        return request_part + Math.round((this.process_items_after_request / this.count_items_export) * after_request_part );
      } */
    /**
     * @protected
     * @param {?} controller
     * @return {?}
     */
    setInitialization(controller) {
        if (!controller) {
            return;
        }
        this.url = controller.getDataUrl();
        this.clazz = controller.getExpectedClass();
        this.keys = controller.getColumns();
        this.server = controller.getBaseUrl();
        this._sort = controller.getSort();
        this.table_title = controller.getTitle();
    }
    /**
     * @protected
     * @return {?}
     */
    preInit() {
        this.generic = new this.clazz;
        this.attributes = this.generic.getRules();
        this.labels = this.generic.getLabels();
        if (!this.keys) {
            this.keys = Object.keys(this.attributes);
        }
        // tslint:disable-next-line: deprecation
        this.columns = Observable.create((o) => {
            o.next(this.attributes);
        });
    }
    /**
     * @protected
     * @return {?}
     */
    onPagerInit() {
        this.initCacheKey();
        if (!this.attributes) {
            this.preInit();
        }
        this.attributesInit();
        this.checkSubscription(this._subscription_activatedRoute_queryParams);
        this._subscription_activatedRoute_queryParams = this.activatedRoute.queryParams.subscribe(params => {
            this.processParams(params);
        });
        this.checkSubscription(this._subscription_available);
        this._subscription_available = this.availableService.change.subscribe((available) => {
            this._available = available;
            this.pageChange(1);
        });
        /* this.checkSubscription(this._subscription_excel_export_start);
        this._subscription_excel_export_start = this.xlsxService.onExportStart.subscribe((is_started: boolean) => {
          if (is_started !== true) {
            return;
          }
          this.modalRefProgressExportXlsx = this.modalService.openModal(this.getModals(), 'ModalProgressExportXlsx');
    
          this.xlsxService.onExportStart.next(false);
          this.export_xlsx_exit = false;
          this.count_items_export = 0;
          this.process_items_after_request = 0;
          this._items_export = [];
    
          setTimeout(() => {
            this.getData(0);
          }, 500);
    
        }) */
        /* this.checkSubscription(this._subscription_excel_export_ready);
        this._subscription_excel_export_ready = this.xlsxService.onExportReady.subscribe((is_ready: boolean) => {
          if(is_ready !== true) {
            return;
          }
          this.xlsxService.onExportReady.next(false);
    
          this.getData(0);
        }) */
    }
    /**
     * @return {?}
     */
    initCacheKey() {
        /** @type {?} */
        const initFilter = (this.initFilter) ? this.initFilter.toString() : '';
        this.cacheKey = PaginationLibComponent.COMPONENT_NAME + initFilter + this.getUrl();
    }
    /**
     * @protected
     * @return {?}
     */
    getUrl() {
        return '';
    }
    /* protected getModals(): ModalBs[] {
        return [
          {
            content: this.modal_progress_export_xlsx,
            id: 'ModalProgressExportXlsx',
            config: {class: 'modal-md', ignoreBackdropClick: true}
          }
        ];
      } */
    /**
     * @protected
     * @param {?} params
     * @return {?}
     */
    processParams(params) {
        this.urlParams = Object.assign([], params);
        /** @type {?} */
        const _params = params[PaginationLibComponent.PARAM_FIELD];
        if (!_params) {
            /** @type {?} */
            const cache = this.storage.readCache(this.cacheKey);
            this.readAndInitParam((cache) ? cache : 'available');
            this.pageChange(this.page);
            return;
        }
        this.readAndInitParam(_params);
        this.getData();
    }
    /**
     * @protected
     * @param {?} params
     * @return {?}
     */
    readAndInitParam(params) {
        if (this.initFilter) {
            params = params + PaginationLibComponent.SEPARATOR_PROPERTY + this.initFilter;
        }
        if (!params) {
            return;
        }
        /** @type {?} */
        const _params = params.split(PaginationLibComponent.SEPARATOR_PROPERTY);
        _params.forEach((param) => {
            /** @type {?} */
            const p = param.split(PaginationLibComponent.SEPARATOR_VALUE);
            /** @type {?} */
            const p_length = p.length;
            switch (p[0]) {
                case PaginationLibComponent.PROPERTY_PAGE:
                    this.page = parseInt(p[1], 10);
                    this.countPage = parseInt(this.storage.readCache(this.cacheKey + 'count'), 10);
                    break;
                case PaginationTheadCellSortingComponent.PARAM_NAME_DIRECTION:
                    this.direction = (typeof p[1] === 'string'
                        && PaginationSortingDirectionEnum[p[1].toLocaleUpperCase()]) ? (/** @type {?} */ (p[1])) : this.direction_default;
                    break;
                case PaginationTheadCellSortingComponent.PARAM_NAME_SORT:
                    this._sort = p[1];
                    break;
                case 'from':
                    this.from = parseInt(p[1], 10);
                    break;
                case 'to':
                    this.to = parseInt(p[1], 10);
                    break;
                /* case 'filter':
                  try {
                    const self = this;
                    const filter_param: string = (p_length > 2) ?
                      p.slice(1, p_length).join(PaginationLibComponent.SEPARATOR_VALUE) : p[1];
                    filter_param.split(PaginationLibComponent.SEPARATOR_MANY_VALUE).forEach(value => {
                      self.genFilterField(value);
                    });
                  } catch (e) {
                    // skip error
                  }
                  break; */
                /*case 'not-available':
                   if (this.available) {
                    if (!(this.generic instanceof Basket) && !(this.generic instanceof User)) {
                      this.page = 1;
                    }
                  }
                  this.availableService.available = this.available;
                  break;
                  */
                default:
                    /** @type {?} */
                    const components = this.paginationService.getAllComponents();
                    /** @type {?} */
                    const components_length = components.length;
                    for (let i = 0; i < components_length; i++) {
                        /** @type {?} */
                        const component = components[i];
                        if (!this.isValidComponent(component)) {
                            continue;
                        }
                        component['updateEventDataByParamItem']({
                            name: p[0],
                            value: p[1]
                        });
                        break;
                    }
            }
        });
        /** @type {?} */
        const convert = new Date();
        /** @type {?} */
        let _f = null;
        /** @type {?} */
        let _t = null;
        if (this.from) {
            convert.setTime(this.from);
            _f = { year: convert.getFullYear(), month: convert.getMonth() + 1, day: convert.getDate() };
        }
        if (this.to) {
            convert.setTime(this.to - 3600 * 24 * 1000);
            _t = { year: convert.getFullYear(), month: convert.getMonth() + 1, day: convert.getDate() };
        }
        this.dateFieldParam = {
            from: _f,
            to: _t
        };
        this.changeUrl();
        debugger;
    }
    /**
     * @protected
     * @return {?}
     */
    genParams() {
        /** @type {?} */
        const params = [];
        if (this.page > 1) {
            params.push(PaginationLibComponent.PROPERTY_PAGE + PaginationLibComponent.SEPARATOR_VALUE + this.page);
        }
        if (this.direction !== this.direction_default) {
            params.push(PaginationTheadCellSortingComponent.PARAM_NAME_DIRECTION + PaginationLibComponent.SEPARATOR_VALUE + this.direction);
        }
        if (this._sort !== this._sort_default) {
            params.push(PaginationTheadCellSortingComponent.PARAM_NAME_SORT + PaginationLibComponent.SEPARATOR_VALUE + this._sort);
        }
        if (this.from) {
            params.push('from' + PaginationLibComponent.SEPARATOR_VALUE + this.from);
        }
        if (this.to) {
            params.push('to' + PaginationLibComponent.SEPARATOR_VALUE + this.to);
        }
        /** @type {?} */
        const components = this.paginationService.getAllComponents();
        /** @type {?} */
        const components_length = components.length;
        for (let i = 0; i < components_length; i++) {
            /** @type {?} */
            const component = components[i];
            if (!this.isValidComponent(component)) {
                continue;
            }
            /** @type {?} */
            const param = component['getParamItem']();
            debugger;
            if (param) {
                params.push(param);
            }
        }
        /* if (this.filter) {
          params.push('filter' + PaginationLibComponent.SEPARATOR_VALUE + this.filter);
        } */
        if (!this.available) {
            params.push('not-available');
        }
        else {
            params.push('available');
        }
        debugger;
        return params.join(PaginationLibComponent.SEPARATOR_PROPERTY);
    }
    /* protected genFilterField(newValue: string) {
        const arr = newValue.split('=');
        const newFilter = [];
        let isNew = true;
    
        if (this.filter) {
          const filters = this.filter.split(PaginationLibComponent.SEPARATOR_MANY_VALUE);
          filters.forEach(value => {
            if (value.search(arr[0]) === 0) {
              isNew = false;
              if (!arr[1]) {
                return;
              }
              value = newValue;
            }
            newFilter.push(value);
          });
        }
        if (isNew) {
          newFilter.push(newValue);
        }
        this.filter = newFilter.length > 0 ? newFilter.join(PaginationLibComponent.SEPARATOR_MANY_VALUE) : undefined;
      } */
    /**
     * @protected
     * @return {?}
     */
    generatorRequestURL( /* export_first?: number */) {
        /** @type {?} */
        const first = (this.page - 1) * this.max /* isNaN(export_first) ? (this.page - 1) * this.max : export_first */;
        /** @type {?} */
        const max = this.max /* isNaN(export_first) ? this.max : this.max_export */;
        /** @type {?} */
        const available = this.available ? 'available&' : '';
        /** @type {?} */
        let _url = this.url + '?' + available + 'first=' + first + '&max=' + max + '&sort=' + this._sort + '&direction=' + this.direction;
        if (this.from) {
            _url += '&from=' + this.from;
        }
        if (this.to) {
            _url += '&to=' + this.to;
        }
        /* if (this.filter) {
              const arr = this.filter.split(PaginationLibComponent.SEPARATOR_MANY_VALUE);
              arr.forEach(value => {
                _url += '&filter_' + value;
              });
            } */
        /** @type {?} */
        const components = this.paginationService.getAllComponents();
        /** @type {?} */
        const components_length = components.length;
        for (let i = 0; i < components_length; i++) {
            /** @type {?} */
            const component = components[i];
            if (!this.isValidComponent(component)) {
                continue;
            }
            _url += component['generateRequestUrl']();
        }
        return _url;
    }
    /**
     * @protected
     * @return {?}
     */
    write() {
        this.storage.writeСache(this.cacheKey, this.urlParams[PaginationLibComponent.PARAM_FIELD]);
        this.storage.setAvailable(this.available);
    }
    /**
     * @param {?} key
     * @param {?} item
     * @return {?}
     */
    get(key, item) {
        /** @type {?} */
        let localized = false;
        /** @type {?} */
        let dynamic = false;
        try {
            try {
                localized = this.attributes[key].format === AttributePaginationFormatEnum.LOCALIZED
                    || this.attributes[key].format === AttributePaginationFormatEnum.LOCALIZED_TEXTAREA;
                dynamic = this.attributes[key].dynamic;
            }
            catch (e) {
                // ignore
            }
            /** @type {?} */
            const arr_key = key.split('.');
            if (key.length > 1) {
                if (localized) {
                    return this.getLocaleName(arr_key, item);
                }
                /** @type {?} */
                const obj = this.executeFunctionByName(arr_key, item);
                if (typeof obj === 'string') {
                    return obj;
                }
                if (typeof obj === 'object') {
                    return 'ERROR TYPE';
                }
            }
            else {
                if (localized) {
                    return this.getLocaleName(arr_key, item);
                }
                return item[key].length === 0 ? 'dashboard.empty' : item[key] + '';
            }
        }
        catch (e) {
            if (dynamic) {
                return '<img width="14px" height="14px" src="/assets/images/spinner_loading_small_v2.gif" />';
            }
            else {
                return 'dashboard.empty';
            }
        }
    }
    /**
     * @param {?} key
     * @param {?} item
     * @return {?}
     */
    getNumber(key, item) {
        /** @type {?} */
        const num = this.get(key, item);
        return num + '';
    }
    /**
     * @param {?} key
     * @param {?} item
     * @return {?}
     */
    getBoolean(key, item) {
        /** @type {?} */
        let a = this.get(key, item);
        if (a === 'true') {
            a = 'boolean.true';
        }
        else if (a === 'false') {
            a = 'boolean.false';
        }
        return a;
    }
    /**
     * @param {?} key
     * @param {?} item
     * @return {?}
     */
    getCurrency(key, item) {
        /** @type {?} */
        const value = this.get(key, item);
        /** @type {?} */
        const a = parseInt(typeof value === 'string' ? value : '', 2);
        if (isNaN(a)) {
            return a;
        }
        return (a / 100) + '';
    }
    /**
     * @param {?} key
     * @param {?} item
     * @return {?}
     */
    getAccordion(key, item) {
        /** @type {?} */
        let value;
        /** @type {?} */
        let field;
        if (key.indexOf('.') >= 0) {
            value = key.split('.');
            field = value[value.length - 1];
            value.pop();
            key = value.join('.');
        }
        return item[key];
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getColumnType(key) {
        return (this.attributes
            && this.attributes[key]
            && this.attributes[key].filter
            && typeof this.attributes[key].filter.type === 'string') ? this.attributes[key].filter.type : this.component_default['EVENT_TYPE'];
    }
    /**
     * @protected
     * @param {?} key
     * @param {?} item
     * @return {?}
     */
    getDate(key, item) {
        /** @type {?} */
        const date = this.get(key, item);
        return parseInt(typeof date === 'string' ? date : '', 2) + this.getLangOffset();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    setData(item) {
    }
    /**
     * @return {?}
     */
    refresh() {
        this.prev = { server: '', url: '' };
        this.getData();
    }
    /**
     * @return {?}
     */
    clearCache() {
        this.storage.writeСache(this.cacheKey, null);
        this.storage.writeСache(this.cacheKey + 'count', null);
        this.prev = { server: '', url: '' };
    }
    /**
     * @protected
     * @return {?}
     */
    getBaseUrl() {
        return this.location.path().split('?')[0];
    }
    /**
     * @protected
     * @param {?} key
     * @return {?}
     */
    getLabels(key) {
        return (typeof key === 'string' && this.labels[key]) ? this.labels[key] : '';
    }
    /**
     * @protected
     * @param {?} key
     * @param {?} item
     * @return {?}
     */
    getLocaleName(key, item) {
        /** @type {?} */
        const el = this.executeFunctionByName(key, item);
        if (typeof el === 'string') {
            return el;
        }
        if (typeof el === 'object') {
            for (const elKey in el) {
                if (el[elKey].language === this.language.getCurrentLanguage()) {
                    return el[elKey].value;
                }
            }
        }
        return '';
    }
    // TODO: костыль чтобы обновлялась дата при смене локализации
    /**
     * @protected
     * @return {?}
     */
    getLangOffset() {
        /** @type {?} */
        const curLang = this.language.getCurrentLanguage();
        return curLang === 'ru' ? 2 : 1;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFilterChange(event) {
        debugger;
        /** @type {?} */
        let page = 1;
        switch (event.type) {
            /* case 'from':
              try {
                const date = new Date(event.data.year, event.data.month - 1, event.data.day);
                this.from = date.getTime();
              } catch (e) {
                return;
              }
              break;
            case 'to':
              try {
                const date = new Date(event.data.year, event.data.month - 1, event.data.day);
                this.to = date.getTime() + 3600 * 24 * 1000;
              } catch (e) {
                return;
              }
              break;
            case 'text':
              try {
                this.genFilterField(event.data);
              } catch (e) {
                return;
              }
              break; */
            case PaginationTheadCellSortingComponent.EVENT_TYPE:
                if (event.data && typeof event.data.key === 'string' && typeof event.data['value'] === 'string') {
                    this._sort = event.data.key;
                    this.direction = event.data['value'];
                    page = this.page;
                }
                break;
            default:
                /** @type {?} */
                const component = this.paginationService.getComponentByEventType(event.type, this.component_default);
                if (this.isValidComponent(component)) {
                    page = component['updateEventData'](event.data);
                }
        }
        this.pageChange(page);
    }
    /**
     * @protected
     * @return {?}
     */
    changeUrl() {
        /** @type {?} */
        const paramNew = this.genParams();
        debugger;
        if (this.urlParams) {
            this.urlParams[PaginationLibComponent.PARAM_FIELD] = paramNew;
        }
        if (!paramNew) {
            delete this.urlParams[PaginationLibComponent.PARAM_FIELD];
        }
        /** @type {?} */
        const par = this.urlParams;
        debugger;
        this.write();
        return this.urlParams;
    }
    /**
     * @param {?} page
     * @return {?}
     */
    pageChange(page) {
        this.countPage_prev = page === 1 ? 1 : this.countPage;
        this.page = page;
        /** @type {?} */
        const param = this.changeUrl();
        debugger;
        this.router.navigate([this.getBaseUrl()], { queryParams: param });
    }
    /**
     * @protected
     * @return {?}
     */
    getData( /* export_first?: number */) {
        /* if(this.export_xlsx_exit && !isNaN(export_first)) {
          return;
        } */
        /* if(this.export_xlsx_exit && !isNaN(export_first)) {
              return;
            } */
        /** @type {?} */
        const serve = this.server;
        /** @type {?} */
        const url = this.generatorRequestURL( /* export_first */);
        if (this.prev && this.prev.url === url && this.prev.server === this.server) {
            return;
        }
        this.prev = { server: this.server, url: url };
        /* if(isNaN(export_first)) {
          // debugger;
          this._items_export = undefined;
          if(this.prev && this.prev.url === url && this.prev.server === this.server) {
            return;
          }
          this.prev = {server: this.server, url: url};
        } else {
          // debugger;
          if(!Array.isArray(this._items_export)) {
            this._items_export = [];
          }
        } */
        this.runRequest(url /* , export_first */);
    }
    /**
     * @protected
     * @param {?} url
     * @return {?}
     */
    runRequest(url /* , export_first?: number */) {
        /* if(this.export_xlsx_exit && !isNaN(export_first)) {
          return;
        } */
        if (this.countPage_prev /* && isNaN(export_first) */) {
            // debugger;
            this.countPage = this.countPage_prev;
        }
        /** @type {?} */
        const self = this;
        this.http
            .server(this.server)
            .loading(LoadingEnum.FULL /* isNaN(export_first) ? LoadingEnum.FULL : LoadingEnum.DISABLED */)
            .cache(false)
            .get(url, (/** @type {?} */ ({
            /**
             * @param {?} items
             * @param {?} response
             * @return {?}
             */
            success(items, response) {
                // debugger;
                // debugger;
                /* if (this.export_xlsx_exit && !isNaN(export_first)) {
                              return;
                            } */
                /** @type {?} */
                let count_items = parseInt(response.headers.get(HeaderEnum.X_TOTAL_COUNT), 2);
                if (isNaN(count_items)) {
                    count_items = 0;
                }
                this.items_length = items.length;
                // debugger;
                /* if(!isNaN(export_first)) {
                  self.count_items_export = count_items;
                  export_first = export_first + self.max_export;
                  self._items_export = self._items_export.concat(items);
                  if(export_first > count_items) {
                    self.xlsxPrepare();
                  } else {
                    self.getData(export_first);
                  }
                  return;
                } */
                self.setData(items);
                self.countPage = count_items;
                self.visiblePagination = self.countPage > self.max;
                if (self.countPage !== count_items) {
                    self.storage.writeСache(self.cacheKey + 'count', self.countPage);
                }
            },
            /**
             * @param {?} err
             * @return {?}
             */
            failed(err) {
            }
        })));
        /** @type {?} */
        const url_arr = url.split('?');
        if (typeof url_arr[1] === 'string') {
            /** @type {?} */
            const args = url_arr[1].split('&');
            /** @type {?} */
            const args_length = args.length;
            /** @type {?} */
            const args_full = [];
            for (let i = 0; i < args_length; i++) {
                /** @type {?} */
                const arg = args[i];
                if (this.initFilter && this.initFilter === arg) {
                    args_full.push(arg);
                    continue;
                }
                // @TODO: develop static method in filter component 'skipParamByTotal'
                // remove request (filter) param to find total items without filters
                if (arg.substring(0, 7) === 'filter_' || arg.substring(0, 5) === 'from=' || arg.substring(0, 3) === 'to=') {
                    continue;
                }
                if (arg.substring(0, 4) === 'max=') {
                    args_full.push('max=1');
                    continue;
                }
                args_full.push(arg);
            }
            url_arr[1] = args_full.join('&');
        }
        this.http
            .server(this.server)
            .loading(LoadingEnum.FULL /* isNaN(export_first) ? LoadingEnum.FULL : LoadingEnum.DISABLED */)
            .cache(false)
            .get(url_arr.join('?'), (/** @type {?} */ ({
            /**
             * @param {?} items
             * @param {?} response
             * @return {?}
             */
            success(items, response) {
                /* if(this.export_xlsx_exit && !isNaN(export_first)) {
                              return;
                            } */
                /** @type {?} */
                const count_items = parseInt(response.headers.get(HeaderEnum.X_TOTAL_COUNT), 2);
                self.countPageWithoutFilters = isNaN(count_items) ? 0 : count_items;
            },
            /**
             * @param {?} err
             * @return {?}
             */
            failed(err) {
                debugger;
            }
        })));
    }
    /* protected xlsxPrepare() {
        if(this.export_xlsx_exit) {
          return;
        }
        if (this.listener) {
          this.listener.onDataUpdate(this._items_export);
        }
      } */
    /* public xlsxExport() {
    
        if(this.export_xlsx_exit || typeof this._items_export === 'undefined') {
          return;
        }
    
        const data: PaginationModel[] = this._items_export;
    
        const items_length: number = this._items_export.length,
          keys_length: number = this.keys.length,
          items: any[] = [],
          empty: string = 'dashboard.empty';
    
        const xlsx_width_cols: ColInfo[] = [];
    
        for(let i = 0; i < items_length; i++) {
          if(this.export_xlsx_exit) {
            return;
          }
          const item: PaginationModel = data[i],
            row = {};
          for(let j = 0; j < keys_length; j++) {
            const key = this.keys[j];
            if(!this.attributes || !key || !this.attributes[key] || !this.attributes[key].format || !this.attributes[key].label ) {
              continue;
            }
            const attribute = this.attributes[key],
              label: string = attribute.label,
              xlsx_width_col: ColInfo = {wch: (!isNaN(attribute.xlsx_width)) ? attribute.xlsx_width : 20},
              label_i18n: string = this.tr.instant(label);
            let format: string = attribute.format;
            if(format.search('date:') === 0) {
              let date_format: string = 'dd-MM-yyyy';
              if(format.substring(0, 6) === 'date::' && format !== AttributeFormatEnum.DATE_D_EEEE_H_MM){
                date_format = format.substring(7);
              }
              xlsx_width_cols.push(xlsx_width_col);
              row[label_i18n] = this.datePipe.transform(this.getDate(key, item), date_format);
              continue;
            }
            if(["string", "status", "address", "custom", "localized", "localizedTextarea"].indexOf(format) >= 0) {
              xlsx_width_cols.push(xlsx_width_col);
              let key_tr: string = this.get(key, item);
              if(!key_tr) {
                key_tr = empty;
              }
              row[label_i18n] = this.tr.instant(key_tr);
              continue;
            }
            switch (format) {
              case 'number':
                xlsx_width_cols.push(xlsx_width_col);
                let key_tr: string = this.getNumber(key, item)
                if(key_tr !== "0" && !key_tr) {
                  key_tr = empty;
                }
                row[label_i18n] = this.tr.instant(key_tr);
                break;
              case 'boolean':
                xlsx_width_cols.push(xlsx_width_col);
                row[label_i18n] = this.tr.instant(this.getBoolean(key, item));
                break;
              case 'currency':
                xlsx_width_cols.push(xlsx_width_col);
                row[label_i18n] = this.tr.instant(this.getCurrency(key, item));
                break;
              case 'accordion':
                let accordion_items: any[] = this.getAccordion(key, item);
    
                if(!Array.isArray(accordion_items)) {
                  accordion_items = [];
                }
                const accordion_length: number = accordion_items.length,
                  accordion_arr: string[] = key.split('.'),
                  accordion_field = (accordion_arr && accordion_arr[1]) ? accordion_arr[1] : '';
                let accordion_items_value: string = '';
                if(accordion_length === 0 || !accordion_field) {
                  xlsx_width_cols.push(xlsx_width_col);
                  row[label_i18n] = this.tr.instant(empty);
                  break;
                }
                for(let k = 0; k < accordion_length; k++) {
                  let accordion_item: any = accordion_items[k];
                  if(!accordion_item || !accordion_item[accordion_field]) {
                    continue;
                  }
                  const accordion_item_value: string = this.language.getLocalizedString(accordion_item[accordion_field]);
                  accordion_items_value += (accordion_items_value) ? ', ' + accordion_item_value : accordion_item_value;
                }
                xlsx_width_cols.push(xlsx_width_col);
                row[label_i18n] = accordion_items_value ? accordion_items_value : empty;
                break;
              default:
    
    
            }
          }
          items.push(row);
          this.process_items_after_request = i;
        }
        const options: XlsxOptions = {cols: xlsx_width_cols};
        let table_name: string = this.tr.instant('dashboard.table');
        if(this.table_title) {
          table_name += '_' + this.tr.instant(this.table_title);
        }
        if(this.export_xlsx_exit) {
          return;
        }
        this.export_xlsx_exit = false;
    
        this.xlsxService.exportAsExcelFile(items, table_name, options);
        this.modalRefProgressExportXlsx.hide();
        this.process_items_after_request = 0;
        this.count_items_export = 0;
        this._items_export = [];
      } */
    /**
     * @protected
     * @param {?} namespaces
     * @param {?} context
     * @return {?}
     */
    executeFunctionByName(namespaces, context) {
        /** @type {?} */
        const func = namespaces.pop();
        for (let i = 0; i < namespaces.length; i++) {
            context = context[namespaces[i]];
        }
        /** @type {?} */
        const result = context[func];
        if (!result) {
            return 'dashboard.empty';
        }
        return result;
    }
    /**
     * @protected
     * @param {?} subscription
     * @return {?}
     */
    checkSubscription(subscription) {
        if (!subscription || typeof subscription.unsubscribe !== 'function') {
            return;
        }
        subscription.unsubscribe();
    }
    /**
     * @protected
     * @return {?}
     */
    attributesInit() {
        if (this.listener && typeof this.listener.onAttributesInit === 'function') {
            this.listener.onAttributesInit();
        }
    }
    /**
     * @return {?}
     */
    getMax() {
        return this.max;
    }
    /**
     * @return {?}
     */
    setConfig() {
        /** @type {?} */
        const config = this.config;
        this.direction = config.direction;
        this.direction_default = config.direction;
        this.max = config.max;
        this._sort = config.sort;
        this._sort_default = config.sort;
    }
    /**
     * @private
     * @param {?} component
     * @return {?}
     */
    isValidComponent(component) {
        return (component
            && typeof component['updateEventData'] === 'function'
            && typeof component['generateRequestUrl'] === 'function'
            && typeof component['updateEventDataByParamItem'] === 'function'
            && typeof component['getParamItem'] === 'function'
            && typeof component['getValueByKey'] === 'function');
    }
}
PaginationLibComponent.PARAM_FIELD = 'param';
PaginationLibComponent.PROPERTY_PAGE = 'page';
PaginationLibComponent.SEPARATOR_VALUE = '_';
PaginationLibComponent.SEPARATOR_PROPERTY = ';';
PaginationLibComponent.SEPARATOR_MANY_VALUE = ',';
PaginationLibComponent.SEPARATOR_FILTER_COLLECTION_ITEM = '=';
PaginationLibComponent.COMPONENT_NAME = 'Pagination';
PaginationLibComponent.propDecorators = {
    clazz: [{ type: Input }],
    url: [{ type: Input }],
    server: [{ type: Input }],
    keys: [{ type: Input }],
    initFilter: [{ type: Input }],
    sort: [{ type: Input }],
    initialization: [{ type: Input }],
    changeColumnsActual: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationTheadCellDirective {
    /**
     * @param {?} viewContainerRef
     */
    constructor(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
}
PaginationTheadCellDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line: directive-selector
                selector: '[pagination-thead-cell-host]',
            },] }
];
/** @nocollapse */
PaginationTheadCellDirective.ctorParameters = () => [
    { type: ViewContainerRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationTheadCellResolverComponent {
    /**
     * @param {?} componentFactoryResolver
     * @param {?} paginationService
     */
    constructor(componentFactoryResolver, paginationService) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.paginationService = paginationService;
        this.changePaginationEvent = new EventEmitter();
        this.subscription_config = this.paginationService.config.subscribe((config_new) => {
            if (!config_new) {
                return;
            }
            this.setConfig(config_new);
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.onInit();
    }
    /**
     * @return {?}
     */
    onInit() {
        this.loadComponent();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.onDestroy();
    }
    /**
     * @return {?}
     */
    onDestroy() {
        this.checkSubscription(this.subscription_pagination_event);
        this.checkSubscription(this.subscription_config);
    }
    /**
     * @return {?}
     */
    loadComponent() {
        /** @type {?} */
        const data = this.data;
        if (!data) {
            return;
        }
        /** @type {?} */
        const event_type = (typeof this.data.event_type === 'string') ? this.data.event_type
            : PaginationTheadCellPlainComponent.EVENT_TYPE;
        /** @type {?} */
        const component = this.paginationService.getComponentByEventType(event_type, PaginationTheadCellPlainComponent);
        /** @type {?} */
        const componentFactory = this.componentFactoryResolver.resolveComponentFactory(component);
        /** @type {?} */
        const viewContainerRef = this.paginationTheadCellHost.viewContainerRef;
        viewContainerRef.clear();
        /** @type {?} */
        const componentRef = viewContainerRef.createComponent(componentFactory);
        /** @type {?} */
        const component_instance = ((/** @type {?} */ (componentRef.instance)));
        component_instance.key = data.key;
        component_instance.title = data.title;
        component_instance.sortable = data.sortable;
        component_instance.direction = data.direction;
        if (typeof data.value !== 'undefined') {
            component_instance.value = data.value;
        }
        this.checkSubscription(this.subscription_pagination_event);
        this.subscription_pagination_event = component_instance.changePaginationEvent.subscribe((event) => {
            debugger;
            this.changePaginationEvent.emit(event);
        });
    }
    /**
     * @protected
     * @param {?} subscription
     * @return {?}
     */
    checkSubscription(subscription) {
        if (!subscription || typeof subscription.unsubscribe !== 'function') {
            return;
        }
        subscription.unsubscribe();
    }
    /**
     * @private
     * @param {?} config
     * @return {?}
     */
    setConfig(config) {
        this.filter_components = (config && Array.isArray(config.filter_components)) ? config.filter_components : [];
    }
}
PaginationTheadCellResolverComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-pagination-thead-cell',
                template: `<ng-template pagination-thead-cell-host></ng-template>`
            }] }
];
/** @nocollapse */
PaginationTheadCellResolverComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: PaginationService }
];
PaginationTheadCellResolverComponent.propDecorators = {
    data: [{ type: Input }],
    changePaginationEvent: [{ type: Output }],
    paginationTheadCellHost: [{ type: ViewChild, args: [PaginationTheadCellDirective,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationLibModule {
    /**
     * @param {?=} options
     * @return {?}
     */
    static forRoot(options) {
        return {
            ngModule: PaginationLibModule,
            providers: [
                AvailableService,
                {
                    provide: PAGINATION_OPTIONS_TOKEN,
                    useValue: options,
                }
            ]
        };
    }
}
PaginationLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    PaginationTheadCellSortingComponent,
                    PaginationTheadCellPlainComponent,
                    PaginationTheadCellResolverComponent,
                    PaginationTheadCellDirective
                ],
                imports: [
                    CommonModule,
                    TranslateModule,
                ],
                entryComponents: [
                    PaginationTheadCellPlainComponent
                ],
                exports: [
                    PaginationTheadCellResolverComponent,
                    PaginationTheadCellSortingComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationModel extends Updatable {
    /**
     * @return {?}
     */
    getRules() {
        return {
            uuid: {
                format: AttributePaginationFormatEnum.VIEW
            },
            whenCreated: {
                format: AttributePaginationFormatEnum.DATE,
                readonly: true,
                xlsx_width: 10
            },
            whenUpdated: {
                format: AttributePaginationFormatEnum.DATE,
                readonly: true,
                xlsx_width: 10
            }
        };
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const AttributePaginationFilterTypeEnum = {
    SELECT: 'select',
    TEXT: 'text',
    DATE: 'date',
    PLAIN: 'plain',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const PaginationSettingTypeEnum = {
    SWITCH_MODE: 'switch_mode',
    AVAILABLE: 'available',
    COLUMNS: 'columns',
    XLSX: 'xlsx',
    REFRESH: 'refresh',
    SWITCH_BASKET_TYPE: 'switch_basket_type',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AttributePaginationFilterTypeEnum, AttributePaginationFormatEnum, AvailableService, PaginationEvent, PaginationLibComponent, PaginationLibModule, PaginationModel, PaginationService, PaginationSettingTypeEnum, PaginationSortingDirectionEnum, PaginationTheadCellBaseComponent, PaginationTheadCellPlainComponent, StaticImplements, PAGINATION_OPTIONS_TOKEN as ɵa, PaginationTheadCellSortingComponent as ɵb, PaginationTheadCellResolverComponent as ɵc, PaginationTheadCellDirective as ɵd };
//# sourceMappingURL=faab-pagination-lib.js.map
